/*
   64x64 mattrix LED routines for STM32F4xx-nucleo

     Copyright (C) by Y.Ishioka
*/
#include "stm32f4xx.h"
#include "matled.h"

#define  DEBUG_MATLED_INIT_TEST   0

void  delay( int  msec );
void  WaitUsec( int usec );
void  WaitNsec( int nsec );
void  syslog( unsigned int prio, const char *format, ... );


/*
 *  read ADC result data
 *    return : == 0xffffffff : fail or not completed
 */
unsigned int  adc_read( void )
{
  unsigned int  result ;

  if( !(ADC1->SR & 0x00000002) ) {
    return  -1 ;
  }

  result = (unsigned int)ADC1->DR ;

  ADC1->CR2  |=  (0x40000000); /* set SWSTART */

  return( result ) ;
}


/*
 * ADC initialize
 */
void  adc_init( void )
{
  /* set clock enable */
  RCC->APB2ENR |= RCC_APB2ENR_ADC1EN ;
  RCC->AHB1ENR |= RCC_AHB1ENR_GPIOCEN ;

  /** for ADC:PC2 **/
  GPIOC->MODER   &= ~(0x00000030);
  GPIOC->MODER   |=  (0x00000030); /* set Analog Mode */
  GPIOC->PUPDR   &= ~(0x00000030);

  ADC1->CR1  =   (0x00000000);
  ADC1->CR2  =   (0x00000001); /* ADC ON */
  ADC1->SQR3 &= ~(0x3fffffff);
  ADC1->SQR3 |=  (12<<0) ;
  ADC1->SQR1 &= ~(0x00ffffff);
  ADC1->SQR1 |=  (0<<20) ;
  ADC1->CR2  |=  (0x40000000); /* set SWSTART */
}


int  btn_read( void )
{
  int  stat = 0 ;
  unsigned int  data ;

  data = GPIOC->IDR ;
  if( (data & (1 << 13)) == 0 ) {
    stat |= 0x01 ;
  }

  return( stat ) ;
}


void  btn_dev_init( void )
{
  /* set clock enable */
  RCC->AHB1ENR |= ( RCC_AHB1ENR_GPIOCEN
                  ) ;

  /** for BTN:PC13 **/
  GPIOC->MODER   &= ~(0x0c000000);
  GPIOC->PUPDR   &= ~(0x0c000000);
  GPIOC->PUPDR   |=  (0x04000000);
}


void  matled_dev_row_clear( void )
{
    int  i, j ;

    i = 0;
    for( j=0 ; j<DEF_DISP_WIDTH ; j++ ) {
      GPIOB->BSRR = 0x0380 << 16 ; /* off R2 G2 B2 */
      GPIOC->BSRR = 0x0070 << 16 ; /* off R1 G1 B1 */

      /* toggle CLK */
      GPIOC->BSRR = 0x1000 ;       /* on  CLK */
      WaitNsec( 100 ) ;
      GPIOC->BSRR = 0x1000 << 16 ; /* off CLK */
    }
    /* OE high */
    GPIOA->BSRR = 0x1000 ;       /* on  OE */

    /* set A,B,C,D,E */
    GPIOC->BSRR = 0x0f00 << 16 ;    /* off A B C D */
    GPIOD->BSRR = 0x0004 << 16 ;    /* off E */
    //WaitNsec( 100 ) ;
    GPIOC->BSRR = 0x0f00 & (i<<8) ; /* set A B C D */
    GPIOD->BSRR = 0x0004 & (i>>2) ; /* set E */
    WaitNsec( 100 ) ;

    /* toggle LATCH */
    GPIOA->BSRR = 0x0800 ;       /* on  LAT */
    WaitNsec( 100 ) ;
    GPIOA->BSRR = 0x0800 << 16 ; /* off LAT */
    GPIOA->BSRR = 0x1000 << 16 ; /* off OE */
}


uint16_t  output_color;

void  matled_dev_output( unsigned char *outbuff, int col, int allcol )
{
  int i, j;

  for( i=0 ; i<16 ; i++ ) {
    for( j=0 ; j<DEF_DISP_WIDTH ; j++ ) {
      GPIOC->BSRR = 0x0070 << 16 ; /* off R1 G1 B1 */
      GPIOB->BSRR = 0x0380 << 16 ; /* off R2 G2 B2 */
      WaitNsec( 100 ) ;

      /* R1 G1 B1 */
      GPIOC->BSRR = 0x0070 & ((*(outbuff+i*allcol+col+j)*output_color) << 4);

      /* R2 G2 B2 */
      GPIOB->BSRR = 0x0380 & ((*(outbuff+(i+16)*allcol+col+j)*output_color) << 7);
      WaitNsec( 100 ) ;

      /* toggle CLK */
      GPIOC->BSRR = 0x1000 ;       /* on  CLK */
      WaitNsec( 100 ) ;
      GPIOC->BSRR = 0x1000 << 16 ; /* off CLK */
    }
    /* OE high */
    GPIOA->BSRR = 0x1000 ;       /* on  OE */

    /* set A,B,C,D,E */
    GPIOC->BSRR = 0x0f00 << 16 ;    /* off A B C D */
    GPIOD->BSRR = 0x0004 << 16 ;    /* off E */
    //WaitNsec( 100 ) ;
    GPIOC->BSRR = 0x0f00 & (i<<8) ; /* set A B C D */
    GPIOD->BSRR = 0x0004 & (i>>2) ; /* set E */
    WaitNsec( 100 ) ;

    /* toggle LATCH */
    GPIOA->BSRR = 0x0800 ;       /* on  LAT */
    WaitNsec( 100 ) ;
    GPIOA->BSRR = 0x0800 << 16 ; /* off LAT */
    GPIOA->BSRR = 0x1000 << 16 ; /* off OE */

    WaitUsec( 10 );
  }

  matled_dev_row_clear();
}

void  matled_dev_output_c( unsigned char *outbuff, int col, int allcol, uint16_t color  )
{
  output_color = color;
  matled_dev_output( outbuff, col, allcol );
}


void  matled_dev_init( void )
{
  /* set clock enable */
  RCC->AHB1ENR |= ( RCC_AHB1ENR_GPIOAEN
                  | RCC_AHB1ENR_GPIOBEN
                  | RCC_AHB1ENR_GPIOCEN
                  | RCC_AHB1ENR_GPIODEN
                  ) ;

  /** for R2:PB7 G2:PB8 B2:PB9 **/
  GPIOB->MODER   &= ~(0x000fc000);
  GPIOB->MODER   |=  (0x00054000);
  GPIOB->OTYPER  &= ~(0x0380);
  GPIOB->OSPEEDR &= ~(0x000fc000);
  GPIOB->OSPEEDR |=  (0x000a8000);
  GPIOB->PUPDR   &= ~(0x000fc000);

  /** for R1:PC4 G1:PC5 B1:PC6 **/
  /** for A:PC8 B:PC9 C:PC10 D:PC11 **/
  /** for CLK:PC12 **/
  /** 1101 1111 0111 0000 **/
  GPIOC->MODER   &= ~(0x03ff3f00);
  GPIOC->MODER   |=  (0x01551500);
  GPIOC->OTYPER  &= ~(0x1f70);
  GPIOC->OSPEEDR &= ~(0x03ff3f00);
  GPIOC->OSPEEDR |=  (0x02aa2a00);
  GPIOC->PUPDR   &= ~(0x03ff3f00);
  //GPIOC->PUPDR   |=  (0x51551500);

  /** for E:PD2 **/
  GPIOD->MODER   &= ~(0x00000030);
  GPIOD->MODER   |=  (0x00000010);
  GPIOD->OTYPER  &= ~(0x0004);
  GPIOD->OSPEEDR &= ~(0x00000030);
  GPIOD->OSPEEDR |=  (0x00000020);
  GPIOD->PUPDR   &= ~(0x00000030);

  /** for LAT:PA11 OE PA12 **/
  GPIOA->MODER   &= ~(0x03c00000);
  GPIOA->MODER   |=  (0x01400000);
  GPIOA->OTYPER  &= ~(0x1800);
  GPIOA->OSPEEDR &= ~(0x03c00000);
  GPIOA->OSPEEDR |=  (0x02800000);
  GPIOA->PUPDR   &= ~(0x03c00000);

  GPIOB->BSRR = 0x0380 << 16 ; /* off R2 G2 B2 */
  GPIOC->BSRR = 0x0070 << 16 ; /* off R1 G1 B1 */
  GPIOC->BSRR = 0x0f00 << 16 ; /* off A B C D */
  GPIOD->BSRR = 0x0004 << 16 ; /* off E */
  GPIOC->BSRR = 0x1000 << 16 ; /* off CLK */
  GPIOA->BSRR = 0x1000 ;       /* on  OE */
  GPIOA->BSRR = 0x0800 ;       /* on  LAT */
  GPIOA->BSRR = 0x0800 << 16 ; /* off LAT */
  GPIOA->BSRR = 0x1000 << 16 ; /* off OE */

#if DEBUG_MATLED_INIT_TEST
 {
  int  i, j ;

  for( i=0 ; i<32 ; i++ ) {
    for( j=0 ; j<DEF_DISP_WIDTH ; j++ ) {

      GPIOB->BSRR = 0x0080 ; /* for R2 G2 B2 */
      GPIOC->BSRR = 0x0010 ; /* for R1 G1 B1 */

      /* toggle CLK */
      GPIOC->BSRR = 0x1000 ;       /* on  CLK */
      WaitNsec( 100 ) ;
      GPIOC->BSRR = 0x1000 << 16 ; /* off CLK */
    }
    /* OE high */
    GPIOA->BSRR = 0x1000 ;       /* on  OE */

    /* set A,B,C,D,E */
    GPIOC->BSRR = 0x0f00 << 16 ;    /* off A B C D */
    GPIOD->BSRR = 0x0004 << 16 ;    /* off E */
    //WaitNsec( 100 ) ;
    GPIOC->BSRR = 0x0f00 & (i<<8) ; /* set A B C D */
    GPIOD->BSRR = 0x0004 & (i>>2) ; /* set E */
    WaitNsec( 100 ) ;

    /* toggle LATCH */
    GPIOA->BSRR = 0x0800 ;       /* on  LAT */
    WaitNsec( 100 ) ;
    GPIOA->BSRR = 0x0800 << 16 ; /* off LAT */
    GPIOA->BSRR = 0x1000 << 16 ; /* off OE */

    delay( 40 );
  }
 }
#endif

  matled_dev_row_clear();
}


void  matled_init( void )
{
  output_color = 0x07;
  matled_dev_init();
  adc_init();
  btn_dev_init();
}
